package main

import "surdeus.su/core/cli/mtool"
//import "surdeus.su/core/biz/inns"
import "surdeus.su/core/ss/urlenc"
import "surdeus.su/core/amo"
import "surdeus.su/core/amo/common"
import "surdeus.su/core/amo/api"
import "log"
import "bufio"
import "strconv"
import "os"
import "fmt"
import "regexp"
import "sync"

import "encoding/json"

const MEPR = api.MaxEntitiesPerRequest

type Tuple struct {
	Lead *amo.Lead
	Contact *amo.Contact
	Company *amo.Company
}

var RegexpINN = regexp.MustCompile("ИНН")

func GetINNSFromFields(
	fields common.CustomFieldsValues,
) common.CustomFieldsValues {
	ret := fields.FilterByNameRegex(RegexpINN)
	/*ret := common.CustomFieldsValues
	for _, innField := range innFields {
		for _, v := range innField.Values {
			inn, err := inns.NewFromStr(
				fmt.Sprintf("%v", v.Value),
			)
			if err != nil {
				continue
			}
			ret = append(ret, inn)
		}
	}*/
	return ret
}

func (t Tuple) HolderINN() HolderINN {
	ret := HolderINN{}
	if t.Company != nil {
		ret.CompanyID = t.Company.ID
		ret.CompanyINNs = GetINNSFromFields(
			t.Company.CustomFieldsValues,
		)
	}
	if t.Lead != nil {
		ret.LeadINNs = GetINNSFromFields(
			t.Lead.CustomFieldsValues,
		)
	}

	if t.Contact != nil {
		ret.ContactINNs = GetINNSFromFields(
			t.Contact.CustomFieldsValues,
		)
	}
	return ret
}


type HolderINN struct {
	CompanyID int `json:"company_id"`
	CompanyINNs common.CustomFieldsValues`json:"company_inns"`
	ContactINNs common.CustomFieldsValues  `json:"contact_inns"`
	LeadINNs common.CustomFieldsValues `json:"lead_inns"`
}

func GetTuples(
	client *amo.Client,
	leadIDs []int,
) ([]Tuple, error) {
	if len(leadIDs) == 0 {
		return nil, nil
	}
	leads, err := client.GetLeads(
		urlenc.Array[int]{
			"id",
			leadIDs,
		},
		urlenc.Value[string]{
			"with",
			"contacts",
		},
	)
	if err != nil {
		return nil, fmt.Errorf("client.GetLeads(...): %s\n", err)
	}

	comIDs := []int{}
	contactIDs := []int{}
	leadToCom := map[int]int{}
	leadToContact := map[int]int{}

	for _, lead := range leads {
		coms := lead.Embedded.Companies
		if len(coms) > 0 {
			com := coms[0]
			leadToCom[lead.ID] = com.ID
			comIDs = append(comIDs, com.ID)
		}
		mainContact, hasMain := lead.Embedded.
			Contacts.GetMain()
		if hasMain {
			leadToContact[lead.ID] = mainContact.ID
			contactIDs = append(contactIDs, mainContact.ID)
		}
	}

	comMap := map[int] *amo.Company{}
	var coms []amo.Company
	if len(comIDs) > 0 {
		coms, err = client.GetCompanies(
			urlenc.Array[int]{
				"id",
				comIDs,
			},
		)
		if err != nil {
			return nil, fmt.Errorf(
				"client.GetCompanies(...): %s\n", err)
		}
		for i := range coms {
			comMap[coms[i].ID] = &coms[i]
		}

	}
	contactMap := map[int] *amo.Contact{}
	var contacts []amo.Contact
	if len(contactIDs) > 0 {
		contacts, err = client.GetContacts(
			urlenc.Array[int]{
				"id",
				contactIDs,
			},
		)
		if err != nil {
			return nil, fmt.Errorf(
				"client.GetContacts(...): %s\n", err)
		}
	}
	for i := range contacts {
		contactMap[contacts[i].ID] = &contacts[i]
	}

	ret := []Tuple{}
	for i := range leads {
		leadID := leads[i].ID
		tup := Tuple{
			Lead: &leads[i],
		}
		contactID, ok := leadToContact[leadID]
		if ok {
			contact, ok := contactMap[contactID]
			if ok {
				tup.Contact = contact
			}
		}
		companyID, ok := leadToCom[leadID]
		if ok {
			company, ok := comMap[companyID]
			if ok {
				tup.Company = company
			}
		}
		ret = append(ret, tup)
	}

	return ret, nil
}

var INN2COM = mtool.T("inn2com").
Func(func(flags *mtool.Flags){
	var (
		secretPath string
		ThreadN int
	)
	flags.StringVar(
		&secretPath,
		"secret",
		"secret.json",
		"path to the AMO API secret file",
		"AMO_SECRET",
	)
	flags.IntVar(&ThreadN, "threads", 5, "amount of threads to do")
	flags.Parse()

	client, err := amo.NewClient(secretPath)
	if err != nil {
		log.Fatalf("amo.NewClient(...): %s\n", err)
	}
	client.API.SetMRPS(9)

	s := bufio.NewScanner(os.Stdin)
	ids := make([]int, 0)
	for s.Scan() {
		txt := s.Text()
		id, err := strconv.Atoi(txt)
		if err != nil {
			log.Printf(
				"strconv.Atoi(%q): %s\n",
				txt, err,
			)
			continue
		}
		ids = append(ids, id)
	}


	threadSize := len(ids)/ThreadN
	var wg sync.WaitGroup
	holderChan := make(chan HolderINN)

	for i := range make([]struct{}, ThreadN) {

		first := i*threadSize
		last := first+threadSize
		if last >= len(ids) {
			last = len(ids)
		}

		wg.Add(1)
		go func(ids []int) {
			defer wg.Done()
			n := len(ids) / MEPR + 1
			for i:=0 ; i<n ; i++ {
				start := i*MEPR
				end := i+MEPR
				if end >= len(ids) {
					end = len(ids)
				}
				tups, err := GetTuples(client, ids[start:end])
				if err != nil {
					log.Printf("GetTuples(...): %s\n", err)
					continue
				}
				for _, tup := range tups {
					holderChan <- tup.HolderINN()
				}
			}
		}(ids[first:last])
	}

	go func() {
		wg.Wait()
		close(holderChan)
	}()

	holders := []HolderINN{}
	for holder := range holderChan {
		holders = append(holders, holder)
	}

	bts, err := json.MarshalIndent(holders, "", "  ")
	if err != nil {
		log.Fatalf("json.Marshal(...): %s\n", err)
	}
	fmt.Printf("%s", bts)
	log.Printf("len(holders) = %d\n", len(holders))
	log.Printf("Made %d requests.", client.API.RequestsMade())

})
